#include <pebble.h>
#include <intersection.h>

static Window *s_main_window;
static Layer *s_canvas_layer;

static void canvas_update_proc(Layer *this_layer, GContext *ctx) {
  //GRect bounds = layer_get_bounds(this_layer);
  
  GRect rect = GRect(25, 15, 55, 70);
  
  graphics_context_set_fill_color(ctx, GColorYellow);
  
  graphics_fill_rect(ctx, rect, 0, GCornerNone);
  
  graphics_context_set_stroke_width(ctx, 1);
  graphics_context_set_stroke_color(ctx, GColorRed);
  
  GPoint line_start = GPoint(50,90);
  GPoint line_end = GPoint(99,10);
  
  graphics_draw_line(ctx, line_start, line_end);
  
  Intersection intersect = rectintersect(line_start, line_end, rect);
  
  if (intersect.success >= INTERSECTION_FULL) {
    APP_LOG(APP_LOG_LEVEL_INFO, "Intersection");
    graphics_context_set_stroke_color(ctx, GColorRed);
    graphics_context_set_stroke_width(ctx, 3);
    graphics_draw_line(ctx, intersect.p1, intersect.p2);
  }
  
}

static void main_window_load(Window *window) {
  Layer *window_layer = window_get_root_layer(window);
  GRect window_bounds = layer_get_bounds(window_layer);

  // Create Layer
  s_canvas_layer = layer_create(GRect(0, 0, window_bounds.size.w, window_bounds.size.h));
  layer_add_child(window_layer, s_canvas_layer);

  // Set the update_proc
  layer_set_update_proc(s_canvas_layer, canvas_update_proc);
}

static void main_window_unload(Window *window) {
  // Destroy Layer
  layer_destroy(s_canvas_layer);
}

static void init(void) {
  // Create main Window
  s_main_window = window_create();
  window_set_window_handlers(s_main_window, (WindowHandlers) {
    .load = main_window_load,
    .unload = main_window_unload,
  });
  window_stack_push(s_main_window, true);
}

static void deinit(void) {
  // Destroy main Window
  window_destroy(s_main_window);
}

int main(void) {
  init();
  app_event_loop();
  deinit();
}
